from enum import Enum

class {{ enum.class_info.name }}(str, Enum):
    {% for key, value in enum.values|dictsort(true) %}
    {{ key }} = "{{ value }}"
    {% endfor %}

    def __str__(self) -> str:
        return str(self.value)
